"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileStream = void 0;
const stream_1 = require("stream");
const util_1 = require("util");
const fs = require("fs");
const path_1 = require("path");
const yauzl = require("yauzl");
const readFile = (0, util_1.promisify)(fs.readFile);
/**
 * Reads a set of XML or zip files and emits the contents downstream
 */
class FileStream extends stream_1.Transform {
    constructor() {
        super({ objectMode: true });
    }
    /**
     * Pop the next file off the list and emit it. If we've got no more files, close the stream
     */
    async _transform(file, encoding, callback) {
        const extension = (0, path_1.parse)(file).ext.toLowerCase();
        if (extension === ".xml") {
            await this.readFile(file);
        }
        else if (extension === ".zip") {
            await this.readZip(file);
        }
        else {
            this.destroy(Error("Unknown file type: " + file));
        }
        callback();
    }
    async readFile(file) {
        console.log("Processing " + file);
        const contents = await readFile(file, "utf8");
        this.push(contents);
    }
    async readZip(file) {
        return new Promise((resolve, reject) => {
            console.log("Processing zip " + file);
            yauzl.open(file, { lazyEntries: true }, (err, zip) => {
                if (err || !zip) {
                    return reject(err);
                }
                zip.readEntry();
                zip.on("close", resolve);
                zip.on("entry", entry => {
                    if (entry.fileName.toLowerCase().endsWith(".xml")) {
                        console.log("Processing " + entry.fileName);
                        zip.openReadStream(entry, async (error, stream) => {
                            if (error || !stream) {
                                console.log(error);
                            }
                            else {
                                try {
                                    this.push(await this.streamToString(stream));
                                }
                                catch (e) {
                                    console.log(e);
                                }
                            }
                            zip.readEntry();
                        });
                    }
                    else {
                        zip.readEntry();
                    }
                });
            });
        });
    }
    streamToString(stream) {
        const chunks = [];
        return new Promise((resolve, reject) => {
            stream.on("data", chunk => chunks.push(chunk));
            stream.on("error", reject);
            stream.on("end", () => resolve(Buffer.concat(chunks).toString("utf8")));
        });
    }
}
exports.FileStream = FileStream;
